@@ -51,6 +51,8 @@ var lists = {};
 // with them.
 function insert(item, msecs) {
   item._idleStart = Date.now();
+  item._monotonicStartTime = Timer.now();
+
   item._idleTimeout = msecs;
 
   if (msecs < 0) return;
@@ -80,12 +82,12 @@ function listOnTimeout() {
 
   debug('timeout callback ' + msecs);
 
-  var now = Date.now();
+  var now = Timer.now();
   debug('now: ' + now);
 
   var first;
   while (first = L.peek(list)) {
-    var diff = now - first._idleStart;
+    var diff = now - first._monotonicStartTime;
     if (diff < msecs) {
       list.start(msecs - diff, 0);
       debug(msecs + ' list wait because diff is ' + diff);
@@ -113,9 +115,15 @@ function listOnTimeout() {
         threw = false;
       } finally {
         if (threw) {
+          // We need to continue processing after domain error handling
+          // is complete, but not by using whatever domain was left over
+          // when the timeout threw its exception.
+          var oldDomain = process.domain;
+          process.domain = null;
           process.nextTick(function() {
             list.ontimeout();
           });
+          process.domain = oldDomain;
         }
       }
     }
@@ -171,6 +179,7 @@ exports.active = function(item) {
       insert(item, msecs);
     } else {
       item._idleStart = Date.now();
+      item._monotonicStartTime = Timer.now();
       L.append(list, item);
     }
   }
@@ -276,15 +285,21 @@ var Timeout = function(after) {
   this._idlePrev = this;
   this._idleNext = this;
   this._idleStart = null;
+  this._monotonicStartTime = null;
   this._onTimeout = null;
   this._repeat = false;
 };
 
 Timeout.prototype.unref = function() {
   if (!this._handle) {
-    var now = Date.now();
-    if (!this._idleStart) this._idleStart = now;
-    var delay = this._idleStart + this._idleTimeout - now;
+
+    var nowMonotonic = Timer.now();
+    if (!this._idleStart || !this._monotonicStartTime) {
+      this._idleStart = Date.now();
+      this._monotonicStartTime = nowMonotonic;
+    }
+
+    var delay = this._monotonicStartTime + this._idleTimeout - nowMonotonic;
     if (delay < 0) delay = 0;
     exports.unenroll(this);
     this._handle = new Timer();
@@ -373,3 +388,114 @@ exports.clearImmediate = function(immediate) {
     process._needImmediateCallback = false;
   }
 };
+
+
+// Internal APIs that need timeouts should use timers._unrefActive isntead of
+// timers.active as internal timeouts shouldn't hold the loop open
+
+var unrefList, unrefTimer;
+
+
+function unrefTimeout() {
+  var now = Timer.now();
+
+  debug('unrefTimer fired');
+
+  var first;
+  while (first = L.peek(unrefList)) {
+    var diff = now - first._monotonicStartTime;
+
+    if (diff < first._idleTimeout) {
+      diff = first._idleTimeout - diff;
+      unrefTimer.start(diff, 0);
+      unrefTimer.when = now + diff;
+      debug('unrefTimer rescheudling for later');
+      return;
+    }
+
+    L.remove(first);
+
+    var domain = first.domain;
+
+    if (!first._onTimeout) continue;
+    if (domain && domain._disposed) continue;
+
+    try {
+      if (domain) domain.enter();
+      var threw = true;
+      debug('unreftimer firing timeout');
+      first._onTimeout();
+      threw = false;
+      if (domain) domain.exit();
+    } finally {
+      if (threw) process.nextTick(unrefTimeout);
+    }
+  }
+
+  debug('unrefList is empty');
+  unrefTimer.when = -1;
+}
+
+
+exports._unrefActive = function(item) {
+  var msecs = item._idleTimeout;
+  if (!msecs || msecs < 0) return;
+  assert(msecs >= 0);
+
+  L.remove(item);
+
+  if (!unrefList) {
+    debug('unrefList initialized');
+    unrefList = {};
+    L.init(unrefList);
+
+    debug('unrefTimer initialized');
+    unrefTimer = new Timer();
+    unrefTimer.unref();
+    unrefTimer.when = -1;
+    unrefTimer.ontimeout = unrefTimeout;
+  }
+
+  var nowDate = Date.now();
+  var nowMonotonicTimestamp = Timer.now();
+
+  item._idleStart = nowDate;
+  item._monotonicStartTime = nowMonotonicTimestamp;
+
+  if (L.isEmpty(unrefList)) {
+    debug('unrefList empty');
+    L.append(unrefList, item);
+
+    unrefTimer.start(msecs, 0);
+    unrefTimer.when = nowMonotonicTimestamp + msecs;
+    debug('unrefTimer scheduled');
+    return;
+  }
+
+  var when = nowMonotonicTimestamp + msecs;
+
+  debug('unrefList find where we can insert');
+
+  var cur, them;
+
+  for (cur = unrefList._idlePrev; cur != unrefList; cur = cur._idlePrev) {
+    them = cur._monotonicStartTime + cur._idleTimeout;
+
+    if (when < them) {
+      debug('unrefList inserting into middle of list');
+
+      L.append(cur, item);
+
+      if (unrefTimer.when > when) {
+        debug('unrefTimer is scheduled to fire too late, reschedule');
+        unrefTimer.start(msecs, 0);
+        unrefTimer.when = when;
+      }
+
+      return;
+    }
+  }
+
+  debug('unrefList append to end');
+  L.append(unrefList, item);
+};
