@@ -28,6 +28,8 @@ var stream = require('stream');
 var assert = require('assert').ok;
 var constants = require('constants');
 
+var Timer = process.binding('timer_wrap').Timer;
+
 var DEFAULT_CIPHERS = 'ECDHE-RSA-AES128-SHA256:AES128-GCM-SHA256:' + // TLS 1.2
                       'RC4:HIGH:!MD5:!aNULL:!EDH';                   // TLS 1.0
 
@@ -445,30 +447,29 @@ CryptoStream.prototype._read = function read(size) {
   }
 
   var bytesRead = 0,
-      start = this._buffer.offset;
+      start = this._buffer.offset,
+      last = start;
   do {
+    assert(last === this._buffer.offset);
     var read = this._buffer.use(this.pair.ssl, out, size - bytesRead);
     if (read > 0) {
       bytesRead += read;
     }
+    last = this._buffer.offset;
 
     // Handle and report errors
     if (this.pair.ssl && this.pair.ssl.error) {
       this.pair.error();
       break;
     }
-
-    // Get NPN and Server name when ready
-    this.pair.maybeInitFinished();
-
-    // `maybeInitFinished()` can emit the 'secure' event which
-    // in turn destroys the connection in case of authentication
-    // failure and sets `this.pair.ssl` to `null`.
   } while (read > 0 &&
            !this._buffer.isFull &&
            bytesRead < size &&
            this.pair.ssl !== null);
 
+  // Get NPN and Server name when ready
+  this.pair.maybeInitFinished();
+
   // Create new buffer if previous was filled up
   var pool = this._buffer.pool;
   if (this._buffer.isFull) this._buffer.create();
@@ -487,16 +488,21 @@ CryptoStream.prototype._read = function read(size) {
 
   if (bytesRead === 0) {
     // EOF when cleartext has finished and we have nothing to read
-    if (this._opposite._finished && this._internallyPendingBytes() === 0) {
+    if (this._opposite._finished && this._internallyPendingBytes() === 0 ||
+        this.pair.ssl && this.pair.ssl.receivedShutdown) {
       // Perform graceful shutdown
       this._done();
 
       // No half-open, sorry!
-      if (this === this.pair.cleartext)
+      if (this === this.pair.cleartext) {
         this._opposite._done();
 
-      // EOF
-      this.push(null);
+        // EOF
+        this.push(null);
+      } else if (!this.pair.ssl || !this.pair.ssl.receivedShutdown) {
+        // EOF
+        this.push(null);
+      }
     } else {
       // Bail out
       this.push('');
@@ -506,16 +512,11 @@ CryptoStream.prototype._read = function read(size) {
     if (this.ondata) {
       this.ondata(pool, start, start + bytesRead);
 
-      // Deceive streams2
-      var self = this;
+      // Force state.reading to set to false
+      this.push('');
 
-      setImmediate(function() {
-        // Force state.reading to set to false
-        self.push('');
-
-        // Try reading more, we most likely have some data
-        self.read(0);
-      });
+      // Try reading more, we most likely have some data
+      this.read(0);
     } else {
       this.push(pool.slice(start, start + bytesRead));
     }
@@ -791,7 +792,7 @@ function onhandshakestart() {
 
   var self = this;
   var ssl = self.ssl;
-  var now = Date.now();
+  var now = Timer.now();
 
   assert(now >= ssl.lastHandshakeTime);
 
@@ -940,6 +941,10 @@ function SecurePair(credentials, isServer, requestCert, rejectUnauthorized,
     /* The Connection may be destroyed by an abort call */
     if (self.ssl) {
       self.ssl.start();
+
+      /* In case of cipher suite failures - SSL_accept/SSL_connect may fail */
+      if (self.ssl && self.ssl.error)
+        self.error();
     }
   });
 }
