@@ -31,6 +31,8 @@
 #include "tree.h"
 #include "uv.h"
 
+#include "util.h"
+
 #if defined(__OpenBSD__) || defined(__MINGW32__) || defined(_MSC_VER)
 # include <nameser.h>
 #else
@@ -261,9 +263,9 @@ static const char* AresErrnoString(int errorno) {
 
 static void SetAresErrno(int errorno) {
   HandleScope scope;
-  Handle<Value> key = String::NewSymbol("errno");
-  Handle<Value> value = String::NewSymbol(AresErrnoString(errorno));
-  Context::GetCurrent()->Global()->Set(key, value);
+  Local<Value> key = String::NewSymbol("_errno");
+  Local<Value> value = String::NewSymbol(AresErrnoString(errorno));
+  node::process->Set(key, value);
 }
 
 
@@ -311,7 +313,7 @@ class QueryWrap {
 
   static void Callback(void *arg, int status, int timeouts,
       unsigned char* answer_buf, int answer_len) {
-    QueryWrap* wrap = reinterpret_cast<QueryWrap*>(arg);
+    QueryWrap* wrap = static_cast<QueryWrap*>(arg);
 
     if (status != ARES_SUCCESS) {
       wrap->ParseError(status);
@@ -324,7 +326,7 @@ class QueryWrap {
 
   static void Callback(void *arg, int status, int timeouts,
       struct hostent* host) {
-    QueryWrap* wrap = reinterpret_cast<QueryWrap*>(arg);
+    QueryWrap* wrap = static_cast<QueryWrap*>(arg);
 
     if (status != ARES_SUCCESS) {
       wrap->ParseError(status);
@@ -491,7 +493,8 @@ class QueryMxWrap: public QueryWrap {
          mx_current = mx_current->next) {
       Local<Object> mx_record = Object::New();
       mx_record->Set(exchange_symbol, String::New(mx_current->host));
-      mx_record->Set(priority_symbol, Integer::New(mx_current->priority));
+      mx_record->Set(priority_symbol,
+                     Integer::New(mx_current->priority));
       mx_records->Set(Integer::New(i++), mx_record);
     }
 
@@ -593,9 +596,12 @@ class QuerySrvWrap: public QueryWrap {
          srv_current = srv_current->next) {
       Local<Object> srv_record = Object::New();
       srv_record->Set(name_symbol, String::New(srv_current->host));
-      srv_record->Set(port_symbol, Integer::New(srv_current->port));
-      srv_record->Set(priority_symbol, Integer::New(srv_current->priority));
-      srv_record->Set(weight_symbol, Integer::New(srv_current->weight));
+      srv_record->Set(port_symbol,
+                      Integer::New(srv_current->port));
+      srv_record->Set(priority_symbol,
+                      Integer::New(srv_current->priority));
+      srv_record->Set(weight_symbol,
+                      Integer::New(srv_current->weight));
       srv_records->Set(Integer::New(i++), srv_record);
     }
 
@@ -605,6 +611,66 @@ class QuerySrvWrap: public QueryWrap {
   }
 };
 
+class QueryNaptrWrap: public QueryWrap {
+ public:
+  int Send(const char* name) {
+    ares_query(ares_channel,
+               name,
+               ns_c_in,
+               ns_t_naptr,
+               Callback,
+               GetQueryArg());
+    return 0;
+  }
+
+ protected:
+  void Parse(unsigned char* buf, int len) {
+    HandleScope scope;
+
+    ares_naptr_reply* naptr_start;
+    int status = ares_parse_naptr_reply(buf, len, &naptr_start);
+
+    if (status != ARES_SUCCESS) {
+      this->ParseError(status);
+      return;
+    }
+
+    Local<Array> naptr_records = Array::New();
+    Local<String> flags_symbol = String::NewSymbol("flags");
+    Local<String> service_symbol = String::NewSymbol("service");
+    Local<String> regexp_symbol = String::NewSymbol("regexp");
+    Local<String> replacement_symbol = String::NewSymbol("replacement");
+    Local<String> order_symbol = String::NewSymbol("order");
+    Local<String> preference_symbol = String::NewSymbol("preference");
+
+    int i = 0;
+    for (ares_naptr_reply* naptr_current = naptr_start;
+         naptr_current;
+         naptr_current = naptr_current->next) {
+
+      Local<Object> naptr_record = Object::New();
+
+      naptr_record->Set(flags_symbol,
+          String::New(reinterpret_cast<char*>(naptr_current->flags)));
+      naptr_record->Set(service_symbol,
+          String::New(reinterpret_cast<char*>(naptr_current->service)));
+      naptr_record->Set(regexp_symbol,
+          String::New(reinterpret_cast<char*>(naptr_current->regexp)));
+      naptr_record->Set(replacement_symbol,
+          String::New(naptr_current->replacement));
+      naptr_record->Set(order_symbol, Integer::New(naptr_current->order));
+      naptr_record->Set(preference_symbol,
+          Integer::New(naptr_current->preference));
+
+      naptr_records->Set(Integer::New(i++), naptr_record);
+    }
+
+    ares_free_data(naptr_start);
+
+    this->CallOnComplete(naptr_records);
+  }
+};
+
 
 class GetHostByAddrWrap: public QueryWrap {
  public:
@@ -675,7 +741,7 @@ static Handle<Value> Query(const Arguments& args) {
   // object reference, causing wrap->GetObject() to return undefined.
   Local<Object> object = Local<Object>::New(wrap->GetObject());
 
-  String::Utf8Value name(args[0]);
+  node::Utf8Value name(args[0]);
 
   int r = wrap->Send(*name);
   if (r) {
@@ -704,7 +770,7 @@ static Handle<Value> QueryWithFamily(const Arguments& args) {
   // object reference, causing wrap->GetObject() to return undefined.
   Local<Object> object = Local<Object>::New(wrap->GetObject());
 
-  String::Utf8Value name(args[0]);
+  node::Utf8Value name(args[0]);
   int family = args[1]->Int32Value();
 
   int r = wrap->Send(*name, family);
@@ -813,10 +879,28 @@ void AfterGetAddrInfo(uv_getaddrinfo_t* req, int status, struct addrinfo* res) {
 }
 
 
+static Handle<Value> IsIP(const Arguments& args) {
+  HandleScope scope;
+
+  String::AsciiValue ip(args[0]);
+  char address_buffer[sizeof(struct in6_addr)];
+
+  if (uv_inet_pton(AF_INET, *ip, &address_buffer).code == UV_OK) {
+    return scope.Close(v8::Integer::New(4));
+  }
+
+  if (uv_inet_pton(AF_INET6, *ip, &address_buffer).code == UV_OK) {
+    return scope.Close(v8::Integer::New(6));
+  }
+
+  return scope.Close(v8::Integer::New(0));
+}
+
+
 static Handle<Value> GetAddrInfo(const Arguments& args) {
   HandleScope scope;
 
-  String::Utf8Value hostname(args[0]);
+  node::Utf8Value hostname(args[0]);
 
   int fam = AF_UNSPEC;
   if (args[1]->IsInt32()) {
@@ -864,11 +948,15 @@ static void Initialize(Handle<Object> target) {
   assert(r == ARES_SUCCESS);
 
   struct ares_options options;
+  memset(&options, 0, sizeof(options));
+  options.flags = ARES_FLAG_NOCHECKRESP;
   options.sock_state_cb = ares_sockstate_cb;
   options.sock_state_cb_data = uv_default_loop();
 
   /* We do the call to ares_init_option for caller. */
-  r = ares_init_options(&ares_channel, &options, ARES_OPT_SOCK_STATE_CB);
+  r = ares_init_options(&ares_channel,
+                        &options,
+                        ARES_OPT_FLAGS | ARES_OPT_SOCK_STATE_CB);
   assert(r == ARES_SUCCESS);
 
   /* Initialize the timeout timer. The timer won't be started until the */
@@ -882,14 +970,19 @@ static void Initialize(Handle<Object> target) {
   NODE_SET_METHOD(target, "queryNs", Query<QueryNsWrap>);
   NODE_SET_METHOD(target, "queryTxt", Query<QueryTxtWrap>);
   NODE_SET_METHOD(target, "querySrv", Query<QuerySrvWrap>);
+  NODE_SET_METHOD(target, "queryNaptr", Query<QueryNaptrWrap>);
   NODE_SET_METHOD(target, "getHostByAddr", Query<GetHostByAddrWrap>);
   NODE_SET_METHOD(target, "getHostByName", QueryWithFamily<GetHostByNameWrap>);
 
   NODE_SET_METHOD(target, "getaddrinfo", GetAddrInfo);
+  NODE_SET_METHOD(target, "isIP", IsIP);
 
-  target->Set(String::NewSymbol("AF_INET"), Integer::New(AF_INET));
-  target->Set(String::NewSymbol("AF_INET6"), Integer::New(AF_INET6));
-  target->Set(String::NewSymbol("AF_UNSPEC"), Integer::New(AF_UNSPEC));
+  target->Set(String::NewSymbol("AF_INET"),
+              Integer::New(AF_INET));
+  target->Set(String::NewSymbol("AF_INET6"),
+              Integer::New(AF_INET6));
+  target->Set(String::NewSymbol("AF_UNSPEC"),
+              Integer::New(AF_UNSPEC));
 
   oncomplete_sym = NODE_PSYMBOL("oncomplete");
 }
