@@ -23,32 +23,28 @@
 #include "ngx-queue.h"
 #include "handle_wrap.h"
 
-#define UNWRAP_NO_ABORT(type)                                               \
-  assert(!args.Holder().IsEmpty());                                         \
-  assert(args.Holder()->InternalFieldCount() > 0);                          \
-  type* wrap =                                                              \
-      static_cast<type*>(args.Holder()->GetPointerFromInternalField(0));
-
 namespace node {
 
-using v8::Array;
-using v8::Object;
-using v8::Handle;
-using v8::Local;
-using v8::Persistent;
-using v8::Value;
-using v8::HandleScope;
-using v8::FunctionTemplate;
-using v8::String;
-using v8::Function;
-using v8::TryCatch;
-using v8::Context;
 using v8::Arguments;
+using v8::Array;
+using v8::Context;
+using v8::Function;
+using v8::FunctionTemplate;
+using v8::Handle;
+using v8::HandleScope;
 using v8::Integer;
+using v8::Local;
+using v8::Object;
+using v8::Persistent;
+using v8::String;
+using v8::TryCatch;
+using v8::Undefined;
+using v8::Value;
 
 
 // defined in node.cc
 extern ngx_queue_t handle_wrap_queue;
+static Persistent<String> close_sym;
 
 
 void HandleWrap::Initialize(Handle<Object> target) {
@@ -61,9 +57,9 @@ Handle<Value> HandleWrap::Ref(const Arguments& args) {
 
   UNWRAP_NO_ABORT(HandleWrap)
 
-  if (wrap) {
+  if (wrap != NULL && wrap->handle__ != NULL) {
     uv_ref(wrap->handle__);
-    wrap->unref_ = false;
+    wrap->flags_ &= ~kUnref;
   }
 
   return v8::Undefined();
@@ -75,9 +71,9 @@ Handle<Value> HandleWrap::Unref(const Arguments& args) {
 
   UNWRAP_NO_ABORT(HandleWrap)
 
-  if (wrap) {
+  if (wrap != NULL && wrap->handle__ != NULL) {
     uv_unref(wrap->handle__);
-    wrap->unref_ = true;
+    wrap->flags_ |= kUnref;
   }
 
   return v8::Undefined();
@@ -91,18 +87,26 @@ Handle<Value> HandleWrap::Close(const Arguments& args) {
       args.Holder()->GetPointerFromInternalField(0));
 
   // guard against uninitialized handle or double close
-  if (wrap && wrap->handle__) {
-    assert(!wrap->object_.IsEmpty());
-    uv_close(wrap->handle__, OnClose);
-    wrap->handle__ = NULL;
+  if (wrap == NULL || wrap->handle__ == NULL) {
+    return Undefined();
   }
 
-  return v8::Null();
+  assert(!wrap->object_.IsEmpty());
+  uv_close(wrap->handle__, OnClose);
+  wrap->handle__ = NULL;
+
+  if (args[0]->IsFunction()) {
+    if (close_sym.IsEmpty() == true) close_sym = NODE_PSYMBOL("close");
+    wrap->object_->Set(close_sym, args[0]);
+    wrap->flags_ |= kCloseCallback;
+  }
+
+  return Undefined();
 }
 
 
 HandleWrap::HandleWrap(Handle<Object> object, uv_handle_t* h) {
-  unref_ = false;
+  flags_ = 0;
   handle__ = h;
   if (h) {
     h->data = this;
@@ -130,6 +134,8 @@ HandleWrap::~HandleWrap() {
 
 
 void HandleWrap::OnClose(uv_handle_t* handle) {
+  HandleScope scope;
+
   HandleWrap* wrap = static_cast<HandleWrap*>(handle->data);
 
   // The wrap object should still be there.
@@ -138,6 +144,11 @@ void HandleWrap::OnClose(uv_handle_t* handle) {
   // But the handle pointer should be gone.
   assert(wrap->handle__ == NULL);
 
+  if (wrap->flags_ & kCloseCallback) {
+    assert(close_sym.IsEmpty() == false);
+    MakeCallback(wrap->object_, close_sym, 0, NULL);
+  }
+
   wrap->object_->SetPointerInInternalField(0, NULL);
   wrap->object_.Dispose();
   wrap->object_.Clear();
