@@ -20,18 +20,30 @@
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 var path = require('path');
+var fs = require('fs');
 var assert = require('assert');
 
 exports.testDir = path.dirname(__filename);
 exports.fixturesDir = path.join(exports.testDir, 'fixtures');
 exports.libDir = path.join(exports.testDir, '../lib');
 exports.tmpDir = path.join(exports.testDir, 'tmp');
-exports.PORT = 12346;
+exports.PORT = +process.env.NODE_COMMON_PORT || 12346;
 
 if (process.platform === 'win32') {
   exports.PIPE = '\\\\.\\pipe\\libuv-test';
+  exports.opensslCli = path.join(process.execPath, '..', 'openssl-cli.exe');
 } else {
   exports.PIPE = exports.tmpDir + '/test.sock';
+  exports.opensslCli = path.join(process.execPath, '..', 'openssl-cli');
+}
+if (!fs.existsSync(exports.opensslCli))
+  exports.opensslCli = false;
+
+if (process.platform === 'win32') {
+  exports.faketimeCli = false;
+} else {
+  exports.faketimeCli = path.join(__dirname, "..", "tools", "faketime", "src",
+    "faketime");
 }
 
 var util = require('util');
@@ -63,6 +75,17 @@ exports.ddCommand = function(filename, kilobytes) {
 };
 
 
+exports.spawnCat = function(options) {
+  var spawn = require('child_process').spawn;
+
+  if (process.platform === 'win32') {
+    return spawn('more', [], options);
+  } else {
+    return spawn('cat', [], options);
+  }
+};
+
+
 exports.spawnPwd = function(options) {
   var spawn = require('child_process').spawn;
 
@@ -90,10 +113,6 @@ process.on('exit', function() {
                       process,
                       global];
 
-  if (global.errno) {
-    knownGlobals.push(errno);
-  }
-
   if (global.gc) {
     knownGlobals.push(gc);
   }
@@ -108,6 +127,14 @@ process.on('exit', function() {
     knownGlobals.push(DTRACE_NET_SOCKET_READ);
     knownGlobals.push(DTRACE_NET_SOCKET_WRITE);
   }
+  if (global.COUNTER_NET_SERVER_CONNECTION) {
+    knownGlobals.push(COUNTER_NET_SERVER_CONNECTION);
+    knownGlobals.push(COUNTER_NET_SERVER_CONNECTION_CLOSE);
+    knownGlobals.push(COUNTER_HTTP_SERVER_REQUEST);
+    knownGlobals.push(COUNTER_HTTP_SERVER_RESPONSE);
+    knownGlobals.push(COUNTER_HTTP_CLIENT_REQUEST);
+    knownGlobals.push(COUNTER_HTTP_CLIENT_RESPONSE);
+  }
 
   if (global.ArrayBuffer) {
     knownGlobals.push(ArrayBuffer);
@@ -144,7 +171,9 @@ process.on('exit', function() {
 var mustCallChecks = [];
 
 
-function runCallChecks() {
+function runCallChecks(exitCode) {
+  if (exitCode !== 0) return;
+
   var failed = mustCallChecks.filter(function(context) {
     return context.actual !== context.expected;
   });
